import numpy as np 

# =================================================================================================
#   Name: 
#       linelist 
#   Purpose: 
#       List of commonly used absorption lines (wl,f,gamma) 
#  Values are from Morton+03, Verner+96, and VPFIT atom file   
# =================================================================================================

wl = {  
        #   HI  Lyman series   

        'HI_1215'  : 1215.6701,  
        'Lya'      : 1215.6701, 
        'HI_1025'  : 1025.7223,  
        'Lyb'      : 1025.7223,      
        'HI_972'   :  972.5368,  
        'Lyg'      :  972.5368,      
        'HI_949'   :  949.7431,  
        'Lyd'      :  949.7431,         
        'HI_937'   :  937.8035,  
        'Lye'      :  937.8035,  
        'HI_930'   :  930.7483,  
        'HI_926'   :  926.2257,  
        'HI_923'   :  923.1504, 
        'HI_920'   :  920.9631, 
        'HI_919'   :  919.3514,  
        'HI_918'   :  918.1294, 
        'HI_917'   :  917.1806, 
        'HI_916'   :  916.429,  
        'HI_915a'  :  915.824,  
        'HI_915b'  :  915.329,   
        'HI_914a'  :  914.919,  
        'HI_914b'  :  914.576,  
        'HI_914c'  :  914.286,  
        'HI_914d'  :  914.039,  
        'HI_913a'  :  913.826,   
        'HI_913b'  :  913.641,   
        'HI_913c'  :  913.480,  
        'HI_913d'  :  913.339,  
        'HI_913e'  :  913.215,  
        'HI_913f'  :  913.104,  
        'HI_913g'  :  913.006,  
        'HI_912a'  :  912.918,  
        'HI_912b'  :  912.839,  
        'HI_912c'  :  912.768,  
        'HI_912d'  :  912.703,   
        'HI_912e'  :  912.645,  
        'HI_911'   :  911.7536, 

        #  HI Balmer Series 

        'Ha' : 6564.623, 
        'Hb' : 4862.688, 
        'Hg' : 4341.690,  

        # Deuterium 

        'DI_1215' :  1215.3394,   
        'DI_1025' :  1025.4433,   
        'DI_972'  :   972.2722,   
        'DI_949'  :   949.4847,   
        'DI_937'  :   937.5484,   
        'DI_930'  :   930.4951,   
        'DI_925'  :   925.9737,   
        'DI_922'  :   922.899,     
        'DI_920'  :   920.712,     
        'DI_919'  :   919.102,     
        'DI_917'  :   917.879,     
        'DI_916a' :   916.931,     
        'DI_916b' :   916.179,     
        'DI_915a' :   915.575,     
        'DI_915b' :   915.080,     

        # Helium 

        #HeI 
        'HeI_584'  :  584.334,   
        'HeI_537'  :  537.0296, 
        'HeI_522'  :  522.2128, 
        'HeI_515'  :  515.6165, 

        #HeII 
        'HeII_303' :  303.7822, 
        'HeII_256' :  256.317, 

        # Carbon  

        # CI 
        'CI_1656'    : 1656.9284, 
        'CI_1560'    : 1560.3092, 
        'CI_1328'    : 1328.8333, 
        'CI_1280'    : 1280.1352, 
        'CI_1260'    : 1260.7351, 
        'CI_1193a'   : 1193.9954, 
        'CI_1193b'   : 1193.0300, 
        'CI_1188'    : 1188.8329, 
        'CI_1157'    : 1157.9097, 
        'CI_945'     :   945.188,  

        # CII 
        'CII_1334'   : 1334.5323,    
        'CII_1036'   : 1036.3367,  
        'CII_903a'   : 903.9616,    
        'CII_903b'   : 903.6235,     
        'CII_858'    : 858.0918,     
        'CII_687'    : 687.0526,     
        'CII_635'    : 635.9945,     
        'CII_594'    : 594.8000,     
        'CII_576'    : 576.8748,     
        'CII_560'    : 560.2394,     
        'CII_551'    : 551.6800,     
        'CII_549'    : 549.3785,     
        'CII_543'    : 543.2570,     
        'CII_530'    : 530.2740,     
        'CII*_1335a' : 1335.7077,  
        'CII*_1335b' : 1335.6627,  
        'CII*_1337'  : 1037.0182,  

        # CIII & CIV 
        'CIII_977'   : 977.0201,   
        'CIII'       : 977.0201,   
        'CIV_1548'   : 1548.204,   
        'CIV'        : 1548.204,   
        'CIV_1550'   : 1550.77755, 

        # Nitrogen 

        # NI 
        'NI_1200a'  : 1200.7098, 
        'NI_1200b'  : 1200.2233, 
        'NI_1199'   : 1199.5496, 
        'NI_1134a'  : 1134.9803, 
        'NI_1134b'  : 1134.4149, 
        'NI_1134c'  : 1134.1653, 
        'NI_965'    : 965.0413,  
        'NI_964'    : 964.6256,  
        'NI_963'    : 963.9903,  
        'NI_954'    : 954.1042,  
        'NI_953a'   : 953.9699,  
        'NI_953b'   : 953.6549,  
        'NI_953c'   : 953.4152,  

        # NII 
        'NII_1083'  : 1083.9937, 
        'NII_915'   : 915.6131, 

        # NIII 
        'NIII_684'  : 684.9960,  
        'NIII_685'  : 685.5130,  
        'NIII_763'  : 763.3400, 
        'NIII_989'  : 989.799,   
        'NIII'      : 989.799, 

        # NIV & NV  
        'NIV_765'   : 765.1480, 
        'NIV'       : 765.1480,  
        'NV_1238'   : 1238.821,  
        'NV'        : 1238.821,  
        'NV_1242'   : 1242.804,       

        # Oxygen    

        # OI 
        'OI_1302'  : 1302.1685,  
        'OI_988'   :  988.7734,  
        'OI_971'   :  971.7382, 

        # OII 
        'OII_834'  :  834.4655,  
        'OII_833'  :  833.3294,  
        'OII_832'  :  832.7572,  

        # OIII 
        'OIII_832' :  832.927,   
        'OIII_702' :  702.332,   

        # OIV 
        'OIV_553'  :  553.3300, 
        'OIV_554'  :  554.0750, 
        'OIV_608'  :  608.3980, 
        'OIV_787'  :  787.711,  
        'OIV'      :  787.711,  

        # OV, OVI, & OVII 
        'OV_629'   :  629.7300, 
        'OV'       :  629.7300, 
        'OVI_1031' : 1031.9261,  
        'OVI'      : 1031.9261,  
        'OVI_1037' : 1037.6167,  
        'OVII_21'  :  21.60169, 
        'OVII_18'  :  18.62799, 

        # Neon

        'NeIV_543' : 543.8910,    
        'NeIV_542' : 542.0730,  
        'NeIV_541' : 541.1270,  
        'NeV_568'  : 568.4200,  
        'NeV'      : 568.4200,  
        'NeVI_558' : 558.5900,  
        'NeVI'     : 558.5900,  
        'NeVIII_780'   : 780.3240,  
        'NeVIII_770'   : 770.4090,  
        'NeVIII'       : 770.4090,  

        # Sodium 

        'NaI_5897'  : 5897.5581,   
        'NaI_5891'  : 5891.5833,  
        'NaI_3303a' :  3303.978,  
        'NaI_3303b' :  3303.369,  
        'NaI_2853'  : 2853.6490,  
        'NaIX_694'  :  694.2700,  
        'NaIX_681'  :  681.7200,  

        # Magnesium 

        'MgI_2026'  : 2026.4768,    
        'MgI_2852'  : 2852.96328,   
        'MgI'       : 2852.96328,   
        'MgII_2803' : 2803.5314853, 
        'MgII_2796' : 2796.3542699, 
        'MgII'      : 2796.3542699, 
        'MgX_624'   : 624.95,       
        'MgX_609'   : 609.79,       

        # Aluminium 

        'AlI_3945'   : 3945.1224,  
        'AlI_3083'   : 3083.0462,  
        'AlI_2367'   : 2367.7750,  
        'AlI_2264'   : 2264.1647,  
        'AlII_1670'  : 1670.7886,  
        'AlII'       : 1670.7886,  
        'AlIII_1862' : 1862.79113, 
        'AlIII_1854' : 1854.71829, 
        'AlIII_696'  : 696.2170,   
        'AlIII_695'  : 695.8289,   
        'AlIII_560a' : 560.4331,   
        'AlIII_560b' : 560.3173,   
        'AlXI_568'   : 568.1500,   
        'AlXI_550'   : 550.0300,   

        # Silicon 

        'SiI_1845'   : 1845.5205,    
        'SiI_1693'   : 1693.2939,    
        'SiI_1631'   : 1631.170,     
        'SiI_1562'   : 1562.002,     
        'SiII_1808'  : 1808.01288,   
        'SiII_1526'  : 1526.70698,   
        'SiII_1304'  : 1304.3702,    
        'SiII_1260'  : 1260.4221,    
        'SiII_1193'  : 1193.2897,    
        'SiII_1190'  : 1190.4158,    
        'SiII_1020'  : 1020.6989,    
        'SiII_989'   : 989.8731,     
        'SiII*_1817' : 1817.4515,    
        'SiII*_1816' : 1816.9285,    
        'SiII*_1533' : 1533.4316,    
        'SiII*_1309' : 1309.2757,    
        'SiII*_1265' : 1265.0020,    
        'SiII*_1264' : 1264.7377,    
        'SiII*_1197' : 1197.3938,    
        'SiII*_1194' : 1194.5002,    
        'SiII*_1023' : 1023.7002,    
        'SiIII_1206' : 1206.500,     
        'SiIII'      : 1206.500,     
        'SiIV_1402'  : 1402.77291,   
        'SiIV_1393'  : 1393.76018,   
        'SiIV'       : 1393.76018,   
        'SiXII_520'  : 520.6700,     
        'SiXII_499'  : 499.4000,     

        # Phosphorus 

        'PI_1774'   : 1774.9487,  
        'PI_1782'   : 1782.8291,  
        'PII_1532'  : 1532.5330,  
        'PII_1301'  : 1301.8743,  
        'PII_1152'  : 1152.8180,  
        'PII_963'   : 963.8005,   
        'PII_961'   : 961.0412,   
        'PIII_1334' : 1334.8132,  
        'PIII_997'  : 997.9996,   
        'PIV_950'   : 950.6569,   
        'PV_1128'   : 1128.0078,  
        'PV_1117'    : 1117.9774,  

        # Sulfur 

        'SI_1807'   : 1807.3113,  
        'SI_1473'   : 1473.9943,  
        'SI_1295'   : 1295.6531,  
        'SII_1259'  : 1259.519,   
        'SII_1253'  : 1253.811,   
        'SII_1250'  : 1250.584,   
        'SIII_1190' : 1190.203,   
        'SIII_1012' : 1012.495,   
        'SIV_1062'  : 1062.664,   
        'SV_786'    : 786.4800,   
        'SV'        : 786.4800,   
        'SVI_944'   : 944.523,    
        'SVI_933'   : 933.378,    
        'SXIV_445'  : 445.7700,   
        'SXIV_417'  : 417.6100,   

        # Chlorine 

        'ClI_1347'  : 1347.2396,  
        'ClI'       : 1347.2396, 

        # Argon 

        'ArI_1066'  : 1066.6598, 
        'ArI_1048'  : 1048.2199, 

        # Calcium

        'CaI_4227'  : 4227.918,  
        'CaI_2276'  : 2276.169,  
        'CaII_3943' : 3934.7750, 
        'CaII_K'    : 3934.7750, 
        'CaII_3969' : 3969.5901, 
        'CaII_H'    : 3969.5901, 
        'CaIX_466'  : 466.2400,  
        'CaIX'      : 466.2400,  
        'CaX_574'   : 574.0100,  
        'CaX_557'   : 557.7630,  
    
        # Titanium 

        'TiII_3384'  : 3384.7304,   
        'TiII_3242'  : 3242.9184,   
        'TiII_3230'  : 3230.1221,   
        'TiII_3073'  : 3073.8633,   
        'TiII_3067'  : 3067.2379,   
        'TiII_1910a' : 1910.9538,   
        'TiII_1910b' : 1910.6123,   

        # Chromium  

        'CrII_2066'  : 2066.16403,  
        'CrII_2062'  : 2062.23610,  
        'CrII_2056'  : 2056.25693,  

        # Manganese 

        'MnII_2606' : 2606.462,   
        'MnII_2594' : 2594.499,   
        'MnII_2576' : 2576.877,   
    
        # Iron      

        'FeI_3021'  : 3021.5187,  
        'FeI_2719'  : 2719.8329,  
        'FeI_2523'  : 2523.6083,  
        'FeI_2484'  : 2484.0209,  
        'FeI_2167'  : 2167.4534,  
        # 
        'FeII_2600' : 2600.1724835,    
        'FeII_2586' : 2586.6495659,   
        'FeII_2382' : 2382.7641781,   
        'FeII_2374' : 2374.4603294,   
        'FeII_2344' : 2344.2129601,   
        'FeII_2260' : 2260.7805,     
        'FeII_1608' : 1608.45085,    
        'FeII_1260' : 1260.533,      
        'FeII_1144' : 1144.9379,     
        'FeII_1143' : 1143.2260,     
        'FeII_1142' : 1142.3656,     
        'FeIII_1112' : 1122.524,     

        # Cobalt 

        'CoII_2059'  : 2059.4756, 
        'CoII_2026'  : 2026.4122, 
        'CoII_2012'  : 2012.1664, 
        'CoII_1941'  : 1941.2852,  
        'CoII_1574'  : 1574.5508, 
        'CoII_1572'  : 1572.6489, 
        'CoII_1552'  : 1552.7624, 
        'CoII_1480'  : 1480.9546, 
        'CoII_1466'  : 1466.2110, 
        'CoII_1424'  : 1424.7866, 

        # Nickel 

        'NiI_2320'   : 2320.7468,   
        'NiI_2311'   : 2311.669,    
        'NiII_1751'  : 1751.9157,   
        'NiII_1741'  : 1741.5531,   
        'NiII_1709'  : 1709.6042,   
        'NiII_1703'  : 1703.4119,   
        'NiII_1502'  : 1502.148,    
        'NiII_1467a' : 1467.756,   
        'NiII_1467b' : 1467.259,   
        'NiII_1454'  : 1454.842,    
        'NiII_1393'  : 1393.324,    
        'NiII_1370'  : 1370.132,    
        'NiII_1317'  : 1317.217,    

        # Zinc 

        'ZnII_2062'  : 2062.66045,    
        'ZnII_2026'  : 2026.13709,   
        
        } 


        ## fosc values 

f = {   
        #   HI  Lyman series   

       'HI_1215'  : 0.416400,   
       'Lya'      : 0.416400,  
       'HI_1025'  : 0.079120,  
       'Lyb'      : 0.079120,  
       'HI_972'   : 0.029000,  
       'Lyg'      : 0.029000,  
       'HI_949'   : 0.013940,  
       'Lyd'      : 0.013940,  
       'HI_937'   : 0.007799,  
       'Lye'      : 0.007799,  
       'HI_930'   : 0.004814,  
       'HI_926'   : 0.003183,  
       'HI_923'   : 0.002216,  
       'HI_920'   : 0.001605,  
       'HI_919'   : 0.00120,   
       'HI_918'   : 0.000921,  
       'HI_917'   : 7.226e-4,  
       'HI_916'   : 0.000577,  
       'HI_915a'  : 0.000469,  
       'HI_915b'  : 0.000386,   
       'HI_914a'  : 0.000321,  
       'HI_914b'  : 0.000270,  
       'HI_914c'  : 0.000230,  
       'HI_914d'  : 0.000197,  
       'HI_913a'  : 0.000170,   
       'HI_913b'  : 0.000148,   
       'HI_913c'  : 0.000129,  
       'HI_913d'  : 0.000114,  
       'HI_913e'  : 0.000101,  
       'HI_913f'  : 0.000089,  
       'HI_913g'  : 0.000080,  
       'HI_912a'  : 0.000071,  
       'HI_912b'  : 0.000064,  
       'HI_912c'  : 0.000058,  
       'HI_912d'  : 0.000053,   
       'HI_912e'  : 0.000048,  
       'HI_911'   : 0.630E-17, 

        #  HI Balmer Series 

        'Ha' : 0.695800,    
        'Hb' : 0.121800, 
        'Hg' : 0.044370,     

        # Deuterium 

        'DI_1215' : 0.416500,   
        'DI_1025' : 0.079100,   
        'DI_972'  : 0.029010,   
        'DI_949'  : 0.013950,   
        'DI_937'  : 0.007808,   
        'DI_930'  : 0.004817,   
        'DI_925'  : 0.003184,   
        'DI_922'  : 0.002216,    
        'DI_920'  : 0.001605,    
        'DI_919'  : 0.001201,    
        'DI_917'  : 0.000921,    
        'DI_916a' : 0.000723,    
        'DI_916b' : 0.000577,    
        'DI_915a' : 0.000469,    
        'DI_915b' : 0.000386,    

        # Helium 

        # HeI 
        'HeI_584'  : 0.285000,   
        'HeI_537'  : 0.0752,   
        'HeI_522'  : 0.0307,   
        'HeI_515'  : 0.0154, 

        # HeII 
        'HeII_303' : 0.416,     
        'HeII_256' : 0.0790,   
        
        # Carbon    

        # CI 
        'CI_1656'    : 0.149000, 
        'CI_1560'    : 0.077400,  
        'CI_1328'    : 0.075800,  
        'CI_1280'    : 0.026300,  
        'CI_1260'    : 0.050700,  
        'CI_1193a'   : 0.012400,  
        'CI_1193b'   : 0.040900,  
        'CI_1188'    : 0.012400,  
        'CI_1157'    : 0.021200,  
        'CI_945'     : 0.2726,    

        # CII 
        'CII_1334'   : 0.127800,       
        'CII_1036'   : 0.118000,    
        'CII_903a'   : 3.360e-01,   
        'CII_903b'   : 1.680e-01,    
        'CII_858'    : 1.300e-02,    
        'CII_687'    : 3.360e-01,    
        'CII_635'    : 6.140e-03,    
        'CII_594'    : 1.170e-01,    
        'CII_576'    : 2.340e-03,    
        'CII_560'    : 5.710e-02,    
        'CII_551'    : 1.120e-03,    
        'CII_549'    : 1.210e-02,    
        'CII_543'    : 3.490e-02,    
        'CII_530'    : 3.620e-02,    
        'CII*_1335a' : 0.115000,    
        'CII*_1335b' : 0.012770,    
        'CII*_1337'  : 0.118000, 

        # CIII & CIV 
        'CIII_977'   : 0.757000,    
        'CIII'       : 0.757000,    
        'CIV_1548'   : 0.189900,    
        'CIV'        : 0.189900,    
        'CIV_1550'   : 0.094750,    


        # Nitrogen 

        # NI 
        'NI_1200a'  : 0.0432,  
        'NI_1200b'  : 0.0869,  
        'NI_1199'   : 0.1320,  
        'NI_1134a'  : 0.0416,  
        'NI_1134b'  : 0.0278,  
        'NI_1134c'  : 0.0146,  
        'NI_965'    : 3.86E-3, 
        'NI_964'    : 7.90e-3, 
        'NI_963'    : 1.24e-2, 
        'NI_954'    : 4.00e-3, 
        'NI_953a'   : 3.31e-2, 
        'NI_953b'   : 2.47e-2, 
        'NI_953c'   : 1.29e-2, 

        # NII 
        'NII_1083'  : 0.111,   
        'NII_915'   : 0.159,   
        
        # NIII 
        'NIII_684'  : 1.350e-01,  
        'NIII_685'  : 2.680e-01,  
        'NIII_763'  : 8.200e-02, 
        'NIII_989'  : 0.123,   
        'NIII'      : 0.123,   

        # NIV & NV 
        'NIV_765'   : 6.160e-01,  
        'NIV'       : 6.160e-01,  
        'NV_1238'   : 0.1560,  
        'NV'        : 0.1560,  
        'NV_1242'   : 0.0770,  

        # Oxygen    

        # OI 
        'OI_1302'  : 0.048000,   
        'OI_988'   : 0.0465,     
        'OI_971'   : 0.0116,    

        # OII 
        'OII_834'  : 0.132,      
        'OII_833'  : 0.0886,     
        'OII_832'  : 0.0444,     

        # OIII 
        'OIII_832' : 0.107,      
        'OIII_702' : 0.137,      

        # OIV 
        'OIV_553'  : 1.120e-01, 
        'OIV_554'  : 2.240e-01, 
        'OIV_608'  : 6.700e-02, 
        'OIV_787'  : 0.110,     
        'OIV'      : 0.110,     

        # OV, OVI, OVII 
        'OV_629'   : 5.150e-1,  
        'OV'       : 5.150e-1,  
        'OVI_1031' : 0.13250,    
        'OVI'      : 0.13250,    
        'OVI_1037' : 0.06580,    
        'OVII_21'  : 0.696,     
        'OVII_18'  : 0.146,     

        # Neon

        'NeIV_543' : 1.160e-01,  
        'NeIV_542' : 7.790e-02, 
        'NeIV_541' : 3.900e-02, 
        'NeV_568'  : 9.280e-02, 
        'NeV'      : 9.280e-02, 
        'NeVI_558' : 9.070e-02,  
        'NeVI'     : 9.070e-02, 
        'NeVIII_780'   : 5.050e-02, 
        'NeVIII_770'   : 1.030e-01, 
        'NeVIII'       : 1.030e-01, 

        # Sodium 

        'NaI_5897'  : 0.3201,      
        'NaI_5891'  : 0.6408,     
        'NaI_3303a' : 0.00460,    
        'NaI_3303b' : 0.00920,    
        'NaI_2853'  : 1.330e-03,  
        'NaIX_694'  : 4.540e-02,  
        'NaIX_681'  : 9.240e-02,  

        # Magnesium 

        'MgI_2026'  : 0.113,     
        'MgI_2852'  : 1.830000,  
        'MgI'       : 1.830000,  
        'MgII_2803' : 0.3058,    
        'MgII_2796' : 0.6155,    
        'MgII'      : 0.6155,    
        'MgX_624'   : 0.0410,    
        'MgX_609'   : 0.0842,     

        # Aluminium 

        'AlI_3945'   : 0.116,       
        'AlI_3083'   : 0.177,       
        'AlI_2367'   : 0.106,       
        'AlI_2264'   : 0.0892,      
        'AlII_1670'  : 1.740,       
        'AlII'       : 1.740,       
        'AlIII_1862' : 0.278,       
        'AlIII_1854' : 0.559,       
        'AlIII_696'  : 4.620e-03,  
        'AlIII_695'  : 9.260e-03,  
        'AlIII_560a' : 2.670e-03,  
        'AlIII_560b' : 5.350e-03,  
        'AlXI_568'   : 3.750e-02,  
        'AlXI_550'   : 7.730e-02,   

        # Silicon 

        'SiI_1845'   : 0.270,  
        'SiI_1693'   : 0.156,      
        'SiI_1631'   : 0.2051,     
        'SiI_1562'   : 0.3758,     
        'SiII_1808'  : 0.00208,    
        'SiII_1526'  : 0.13300,    
        'SiII_1304'  : 0.0863,     
        'SiII_1260'  : 1.180,      
        'SiII_1193'  : 0.5820,     
        'SiII_1190'  : 0.2920,     
        'SiII_1020'  : 0.0168,     
        'SiII_989'   : 0.171,      
        'SiII*_1817' : 1.29E-4,    
        'SiII*_1816' : 1.66E-3,    
        'SiII*_1533' : 0.13200,    
        'SiII*_1309' : 0.0860,     
        'SiII*_1265' : 0.117,      
        'SiII*_1264' : 1.050,      
        'SiII*_1197' : 0.145,      
        'SiII*_1194' : 0.727,      
        'SiII*_1023' : 0.0168,     
        'SiIII_1206' : 1.63000,    
        'SiIII'      : 1.63000,    
        'SiIV_1402'  : 0.254,      
        'SiIV_1393'  : 0.513,      
        'SiIV'       : 0.513,      
        'SiXII_520'  : 3.45e-02,  
        'SiXII_499'  : 7.19e-02,  

        # Phosphorus 

        'PI_1774'   : 0.169,    
        'PI_1782'   : 0.113,    
        'PII_1532'  : 3.03e-3,  
        'PII_1301'  : 1.27e-2, 
        'PII_1152'  : 0.245,   
        'PII_963'   : 1.458,   
        'PII_961'   : 0.3489,  
        'PIII_1334' : 0.0282,   
        'PIII_997'  : 0.1117,   
        'PIV_950'   : 1.490,    
        'PV_1128'   : 0.233,    
        'PV_1117'    : 0.472,    

        # Sulfur 

        'SI_1807'   : 0.0905,       
        'SI_1473'   : 0.0828,       
        'SI_1295'   : 0.0870,       
        'SII_1259'  : 0.0166,       
        'SII_1253'  : 0.0109,       
        'SII_1250'  : 0.00543,      
        'SIII_1190' : 0.0237,       
        'SIII_1012' : 0.0438,       
        'SIV_1062'  : 0.0494,       
        'SV_786'    : 1.420e+00,   
        'SV'        : 1.420e+00,  
        'SVI_944'   : 0.215,        
        'SVI_933'   : 0.437,        
        'SXIV_445'  : 2.990e-02,   
        'SXIV_417'  : 6.390e-02,   

        # Chlorine 

        'ClI_1347'  : 0.153,     
        'ClI'       : 0.153,     

        # Argon 

        'ArI_1066'  : 0.0675,    
        'ArI_1048'  : 0.263,      

        # Calcium

        'CaI_4227'  : 1.770,      
        'CaI_2276'  : 0.0661,     
        'CaII_3943' : 0.6267,     
        'CaII_K'    : 0.6267,     
        'CaII_3969' : 0.3116,     
        'CaII_H'    : 0.3116,     
        'CaIX_466'  : 1.090e+00, 
        'CaIX'      : 1.090e+00, 
        'CaX_574'   : 1.660e-01, 
        'CaX_557'   : 3.430e-01, 
    
        # Titanium 

        'TiII_3384'  : 0.358,   
        'TiII_3242'  : 0.232,   
        'TiII_3230'  : 0.0687,  
        'TiII_3073'  : 0.121,   
        'TiII_3067'  : 0.0489,  
        'TiII_1910a' : 0.0980,  
        'TiII_1910b' : 0.104,   

        # Chromium  

        'CrII_2066'  : 0.0512,  
        'CrII_2062'  : 0.0759,   
        'CrII_2056'  : 0.1030,    

        # Manganese 

        'MnII_2606' : 0.198,   
        'MnII_2594' : 0.280,   
        'MnII_2576' : 0.361,   
    
        # Iron      

        'FeI_3021'  : 0.1040,    
        'FeI_2719'  : 0.122,     
        'FeI_2523'  : 0.203,     
        'FeI_2484'  : 0.544,     
        'FeI_2167'  : 0.150000,  
        # 
        'FeII_2600' : 0.2394,       
        'FeII_2586' : 0.069125,    
        'FeII_2382' : 0.320,       
        'FeII_2374' : 0.0313,      
        'FeII_2344' : 0.1142,      
        'FeII_2260' : 0.00244,    
        'FeII_1608' : 0.0577,     
        'FeII_1260' : 0.024000,   
        'FeII_1144' : 0.0830,     
        'FeII_1143' : 0.0192,     
        'FeII_1142' : 0.00401,    
        'FeIII_1112' :0.05440,    

        # Cobalt 

        'CoII_2059'  : 0.00751,   
        'CoII_2026'  : 0.0125,    
        'CoII_2012'  : 0.0368,    
        'CoII_1941'  : 0.0340,     
        'CoII_1574'  : 0.0250,    
        'CoII_1572'  : 0.0120,    
        'CoII_1552'  : 0.0116,    
        'CoII_1480'  : 0.0119,   
        'CoII_1466'  : 0.0310,   
        'CoII_1424'  : 0.0109,   

        # Nickel 

        'NiI_2320'   : 0.685,      
        'NiI_2311'   : 0.376500,   
        'NiII_1751'  : 0.027700,   
        'NiII_1741'  : 0.042700,   
        'NiII_1709'  : 0.0324,     
        'NiII_1703'  : 0.01224,    
        'NiII_1502'  : 0.00600,    
        'NiII_1467a' : 0.00990,   
        'NiII_1467b' : 0.00630,   
        'NiII_1454'  : 0.0323,     
        'NiII_1393'  : 0.0101,     
        'NiII_1370'  : 0.0769,     
        'NiII_1317'  : 0.0571,     

        # Zinc 

        'ZnII_2062'  : 0.246,   
        'ZnII_2026'  : 0.501, 

        } 

        ## Gamma values 

gamma = {   

        #   HI  Lyman series    

       'HI_1215'  : 6.265E8,                   
       'Lya'      : 6.265E8, 
       'HI_1025'  : 1.897E8, 
       'Lyb'      : 1.897E8, 
       'HI_972'   : 8.127E7, 
       'Lyg'      : 8.127E7, 
       'HI_949'   : 4.204E7, 
       'Lyd'      : 4.204E7, 
       'HI_937'   : 2.450E7, 
       'Lye'      : 2.450E7, 
       'HI_930'   : 1.236E7, 
       'HI_926'   : 8.255E6, 
       'HI_923'   : 5.785E6,
       'HI_920'   : 4.210E6, 
       'HI_919'   : 3.160E6, 
       'HI_918'   : 2.432E6, 
       'HI_917'   : 1.911E6, 
       'HI_916'   : 1.529E6, 
       'HI_915a'  : 1.243E6, 
       'HI_915b'  : 1.024E6,  
       'HI_914a'  : 8.533E5, 
       'HI_914b'  : 7.186E5, 
       'HI_914c'  : 6.109E5, 
       'HI_914d'  : 5.237E5,  
       'HI_913a'  : 4.523E5,  
       'HI_913b'  : 3.933E5,  
       'HI_913c'  : 3.443E5,
       'HI_913d'  : 3.030E5,
       'HI_913e'  : 2.679E5,
       'HI_913f'  : 2.382E5,
       'HI_913g'  : 2.127E5,
       'HI_912a'  : 1.907E5,
       'HI_912b'  : 1.716E5,
       'HI_912c'  : 1.550E5,
       'HI_912d'  : 1.405E5,  
       'HI_912e'  : 1.277E5, 
       'HI_911'   : 1.0000,   
        
       #  HI Balmer Series 

        'Ha' : 6.465E7,    
        'Hb' : 2.062E7, 
        'Hg' : 9.425E6,    

        # Deuterium 

        'DI_1215' : 6.270E8,  
        'DI_1025' : 1.897E8,  
        'DI_972'  : 8.127E7,  
        'DI_949'  : 4.204E7,  
        'DI_937'  : 2.450E7,  
        'DI_930'  : 1.237E7,  
        'DI_925'  : 8.261E6,  
        'DI_922'  : 5.789E6,   
        'DI_920'  : 4.214E6,   
        'DI_919'  : 3.162E6,   
        'DI_917'  : 2.434E6,   
        'DI_916a' : 1.913E6,   
        'DI_916b' : 1.531E6,   
        'DI_915a' : 1.244E6,   
        'DI_915b' : 1.025E6,  

        # Helium 

        'HeI_584'  : 2.000e8,  
        'HeI_537'  : 2.000e8,
        'HeI_522'  : 2.000e8,
        'HeI_515'  : 2.000E8,
        'HeII_303' : 6.270E8,  
        'HeII_256' : 1.897E8, 

        
        # Carbon    

        # CI 
        'CI_945'     : 1.00e8,   
        'CI_1157'    : 3.510e7,  
        'CI_1188'    : 1.950E7,  
        'CI_1193a'   : 1.940E7, 
        'CI_1193b'   : 6.390E7,  
        'CI_1260'    : 2.400e8,  
        'CI_1280'    : 1.060E8,  
        'CI_1328'    : 2.880E8,  
        'CI_1560'    : 1.270e8,  
        'CI_1656'    : 3.600e8,  
        
        # CII 
        'CII_1334'   : 2.880e8,     
        'CII_1036'   : 2.200e9, 
        'CII_903a'   : 1.0000, 
        'CII_903b'   : 1.0000,  
        'CII_858'    : 1.0000,  
        'CII_687'    : 1.0000,  
        'CII_635'    : 1.0000,  
        'CII_594'    : 1.0000,  
        'CII_576'    : 1.0000,  
        'CII_560'    : 1.0000,  
        'CII_551'    : 1.0000,  
        'CII_549'    : 1.0000,  
        'CII*_1335a' : 2.880e8,  
        'CII*_1335b' : 2.880e8,  
        'CII*_1337'  : 2.200e9, 

        # CIII & CIV   
        'CIII_977'   : 1.760E9,  
        'CIII'       : 1.760E9,  
        'CIV_1548'   : 2.642E8,  
        'CIV'        : 2.642E8,  
        'CIV_1550'   : 2.628E8,  

        # Nitrogen 

        # NI 
        'NI_1200a'  : 4.00e8,  
        'NI_1200b'  : 4.02e8,  
        'NI_1199'   : 4.07e8,  
        'NI_1134a'  : 1.44e8,  
        'NI_1134b'  : 1.49e8,  
        'NI_1134c'  : 1.51e8,  
        'NI_965'    : 8.16e7,  
        'NI_964'    : 8.31e7,  
        'NI_963'    : 8.55e7,  
        'NI_954'    : 1.41e8,  
        'NI_953a'   : 2.03e8,  
        'NI_953b'   : 2.10e8,  
        'NI_953c'   : 2.19e8,  

        # NII 
        'NII_1083'  : 3.74e8,  
        'NII_915'   : 1.27e9, 

        # NIII 
        'NIII_684'  : 1.0000, 
        'NIII_685'  : 1.0000, 
        'NIII_763'  : 1.0000, 
        'NIII_989'  : 5.00e8,  
        'NIII'      : 5.00e8,  

        # NIV & NV 
        'NIV_765'   : 1.0000,   
        'NIV'       : 1.0000,   
        'NV_1238'   : 3.391e8, 
        'NV'        : 3.391e8, 
        'NV_1242'   : 3.356e8,  

        # Oxygen    

        # OI 
        'OI_1302'  : 5.650e8,  
        'OI_988'   : 2.26E8,   
        'OI_971'   : 8.58e7,  

        # OII 
        'OII_834'  : 2.000E8,  
        'OII_833'  : 2.00E8,   
        'OII_832'  : 2.00E8,   

        # OIII 
        'OIII_832' : 2.000E8,  
        'OIII_702' : 2.000E8,  

        # OIV 
        'OIV_553'  : 1.0000, 
        'OIV_554'  : 1.0000, 
        'OIV_608'  : 1.0000,
        'OIV_787'  : 2.000E8, 
        'OIV'      : 2.000E8,  

        # OV, OVI, OVII  
        'OV_629'   : 1.00000,  
        'OV'       : 1.00000,  
        'OVI_1031' : 4.149e8,   
        'OVI'      : 4.149e8,   
        'OVI_1037' : 4.076e8,   
        'OVII_21'  : 3.32E12, 
        'OVII_18'  : 9.35E11, 

        # Neon

        'NeIV_543' : 1.0000,  
        'NeIV_542' : 1.0000, 
        'NeIV_541' : 1.0000, 
        'NeV_568'  : 1.0000, 
        'NeV'      : 1.0000, 
        'NeVI_558' : 1.0000, 
        'NeVI'     : 1.0000, 
        'NeVIII_780'   : 1.0000, 
        'NeVIII_770'   : 1.0000, 
        'NeVIII'       : 1.0000, 

        # Sodium 

        'NaI_5897'  : 6.139e7,  
        'NaI_5891'  : 6.157e7, 
        'NaI_3303a' : 9.639e6, 
        'NaI_3303b' : 9.641e6, 
        'NaI_2853'  : 1.0000, 
        'NaIX_694'  : 1.0000, 
        'NaIX_681'  : 1.0000, 

        # Magnesium 

        'MgI_2026'  :  6.12e7,  
        'MgI_2852'  :  5.000e8, 
        'MgI'       :  5.000e8, 
        'MgII_2803' :  2.66e8,  
        'MgII_2796' :  2.68e8,  
        'MgII'      :  2.68e8,  
        'MgX_624'   :  2.000E8, 
        'MgX_609'   :  2.000E8, 

        # Aluminium 

        'AlI_3945'   : 1.48e8,  
        'AlI_3083'   : 7.45e7,  
        'AlI_2367'   : 7.58e7,  
        'AlI_2264'   : 7.14e7,  
        'AlII_1670'  : 1.390e9, 
        'AlII'       : 1.390e9, 
        'AlIII_1862' : 5.34e8,  
        'AlIII_1854' : 5.42e8,  
        'AlIII_696'  : 1.0000, 
        'AlIII_695'  : 1.0000, 
        'AlIII_560a' : 1.0000, 
        'AlIII_560b' : 1.0000, 
        'AlXI_568'   : 1.0000, 
        'AlXI_550'   : 1.0000,  

        # Silicon 

        'SiI_1845'   : 1.76e8,  
        'SiI_1693'   : 1.21e8,  
        'SiI_1631'   : 1.714e8, 
        'SiI_1562'   : 3.425e8, 
        'SiII_1808'  : 2.38e6,  
        'SiII_1526'  : 1.13E9,  
        'SiII_1304'  : 1.01E9,  
        'SiII_1260'  : 2.95E9,  
        'SiII_1193'  : 4.07e9,  
        'SiII_1190'  : 4.08E9,  
        'SiII_1020'  : 6.680e8, 
        'SiII_989'   : 6.673e8, 
        'SiII*_1817' : 2.38E6,  
        'SiII*_1816' : 2.24E6,  
        'SiII*_1533' : 1.13E9,  
        'SiII*_1309' : 1.01E9,  
        'SiII*_1265' : 2.95E9,  
        'SiII*_1264' : 2.93E9,  
        'SiII*_1197' : 4.07e9,  
        'SiII*_1194' : 4.08E9,  
        'SiII*_1023' : 2.13E8,  
        'SiIII_1206' : 2.48e9,  
        'SiIII'      : 2.48e9,  
        'SiIV_1402'  : 8.62e8,  
        'SiIV_1393'  : 8.80e8,  
        'SiIV'       : 8.80e8,  
        'SiXII_520'  : 1.0000, 
        'SiXII_499'  : 1.0000, 

        # Phosphorus 

        'PI_1774'   : 2.38e8,  
        'PI_1782'   : 2.38e8,  
        'PII_1532'  : 4.77e6,  
        'PII_1301'  : 4.82e7, 
        'PII_1152'  : 1.23e9, 
        'PII_963'   : 4.14E9, 
        'PII_961'   : 5.09e9, 
        'PIII_1334' : 6.32E7,  
        'PIII_997'  : 2.220E9, 
        'PIV_950'   : 3.67e9,  
        'PV_1128'   : 1.22e9,  
        'PV_1117'    : 1.26e9,  

        # Sulfur 

        'SI_1807'   : 5.33e8,  
        'SI_1473'   : 1.82e8,  
        'SI_1295'   : 3.46e8,  
        'SII_1259'  : 4.65e7,  
        'SII_1253'  : 4.62e7,  
        'SII_1250'  : 4.63e7,  
        'SIII_1190' : 6.65e7,  
        'SIII_1012' : 2.81e8,  
        'SIV_1062'  : 1.69e8,  
        'SV_786'    : 1.0000, 
        'SV'        : 1.0000, 
        'SVI_944'   : 1.61e9,   
        'SVI_933'   : 1.67e9,   
        'SXIV_445'  : 1.0000,  
        'SXIV_417'  : 1.0000,  

        # Chlorine 

        'ClI_1347'  : 6.62e8,  
        'ClI'       : 6.62e8,   

        # Argon 

        'ArI_1066'  : 1.32e8,  
        'ArI_1048'  : 3.18e7,   

        # Calcium

        'CaI_4227'  : 2.200e8, 
        'CaI_2276'  : 2.84e7,  
        'CaII_3943' : 1.444e8, 
        'CaII_K'    : 1.444e8, 
        'CaII_3969' : 1.409e8, 
        'CaII_H'    : 1.409e8, 
        'CaIX_466'  : 1.0000, 
        'CaIX'      : 1.0000, 
        'CaX_574'   : 1.0000, 
        'CaX_557'   : 1.0000, 
    
        # Titanium 

        'TiII_3384'  : 1.75e8,  
        'TiII_3242'  : 2.44e8,  
        'TiII_3230'  : 2.44e8,  
        'TiII_3073'  : 2.56e8,  
        'TiII_3067'  : 2.500e8, 
        'TiII_1910a' : 1.43E8,  
        'TiII_1910b' : 4.17E8,  

        # Chromium  

        'CrII_2066'  : 4.17e8,  
        'CrII_2062'  : 4.06e8,  
        'CrII_2056'  : 4.07e8,  

        # Manganese 

        'MnII_2606'  : 2.72e8,  
        'MnII_2594'  : 2.78e8,  
        'MnII_2576'  : 2.82e8,  
    
        # Iron      

        'FeI_3021'  : 1.69E8,  
        'FeI_2719'  : 1.75E8,  
        'FeI_2523'  : 3.85E8,  
        'FeI_2484'  : 5.000E8, 
        'FeI_2167'  : 2.74E8,  
        # 
        'FeII_2600' : 2.70e8,    
        'FeII_2586' : 2.72E8,   
        'FeII_2382' : 3.13E8,   
        'FeII_2374' : 3.09E8,   
        'FeII_2344' : 2.680e8,  
        'FeII_2260' : 2.58e8,  
        'FeII_1608' : 2.74e8,  
        'FeII_1260' : 1.26E8,  
        'FeII_1144' : 3.520E8, 
        'FeII_1143' : 9.81e7,  
        'FeII_1142' : 2.56e7,  
        'FeIII_1112' : 3.70e8,  

        # Cobalt 

        'CoII_2059'  : 2.78e8,  
        'CoII_2026'  : 3.03e8,  
        'CoII_2012'  : 3.45e8,  
        'CoII_1941'  : 4.35e8,   
        'CoII_1574'  : 6.73e7,  
        'CoII_1572'  : 4.16e7,  
        'CoII_1552'  : 3.21e7,  
        'CoII_1480'  : 4.65e7,  
        'CoII_1466'  : 3.78e7,  
        'CoII_1424'  : 3.58e7,  

        # Nickel 

        'NiI_2320'   : 6.94E8,   
        'NiI_2311'   : 4.700E8,  
        'NiII_1751'  : 3.70E8,   
        'NiII_1741'  : 5.00E8,   
        'NiII_1709'  : 4.35e8,   
        'NiII_1703'  : 5.00E8,   
        'NiII_1502'  : 3.932E7,  
        'NiII_1467a' : 2.30E7,  
        'NiII_1467b' : 2.93E7,  
        'NiII_1454'  : 1.02E8,   
        'NiII_1393'  : 3.47E7,   
        'NiII_1370'  : 4.10e8,   
        'NiII_1317'  : 4.205e8,  

        # Zinc 

        'ZnII_2062'  : 3.86e8,   
        'ZnII_2026'  : 4.07e8, 
        
        }

